"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function generateWebStyle(style, screen, classes) {
    var css = generateCleanStyle(style.large, screen, classes) + "\n";
    if (style.medium) {
        var code = generateCleanStyle(style.medium, screen, classes);
        if (code && code.trim() !== '') {
            css =
                css +
                    ("@media only screen and (max-width: 1024px)  {\n            " + code + "\n        }\n");
        }
    }
    if (style.small) {
        var code = generateCleanStyle(style.small, screen, classes);
        if (code && code.trim() !== '') {
            css =
                css +
                    ("@media only screen and (max-width: 540px)  {\n                " + code + "\n            }\n");
        }
    }
    return css;
}
exports.generateWebStyle = generateWebStyle;
function generateInlineReactCss(styles, screen, classes) {
    var csscode = '';
    for (var k in styles) {
        var key = k;
        if (classes[key] != undefined && classes[key] != '') {
            key = classes[key];
        }
        if (screen.find("." + key)[0]) {
            csscode += "const " + key + " {\n";
            csscode += getCss(styles[key]);
            csscode += '}\n';
        }
    }
    return csscode;
}
exports.generateInlineReactCss = generateInlineReactCss;
function generateStyledReactCss(styles, screen, classes) {
    var csscode = '';
    for (var k in styles) {
        var key = k;
        if (classes[key] != undefined && classes[key] != '') {
            key = classes[key];
        }
        if (screen.find("." + key)[0]) {
            var tag = screen.find("." + key)[0].tagName.toLowerCase();
            csscode += "const " + key + " =styled." + tag + "`{\n";
            csscode += getCss(styles[key]);
            csscode += '}`;\n';
        }
    }
    return csscode;
}
exports.generateStyledReactCss = generateStyledReactCss;
function generateCleanStyle(styles, screen, classes) {
    var valueKeys = removeDupStyles(styles);
    var csscode = '';
    var keys = [];
    for (var v in valueKeys) {
        var key = valueKeys[v];
        if (keys.indexOf(key) == -1) {
            var style = JSON.parse(v);
            if (key.indexOf(',') > -1) {
                var k = key.split(',')[0];
                if (classes[k] != undefined && classes[k] != '') {
                    key = classes[k];
                }
            }
            else if (classes[key] != undefined && classes[key] != '') {
                key = classes[key];
            }
            if (screen.find("." + key)[0]) {
                csscode += "." + key + " {\n";
                csscode += getCss(style);
                csscode += '}\n';
                keys.push(key);
            }
        }
    }
    return csscode;
}
function getCss(elements) {
    var css = '';
    for (var key in elements) {
        if (key && elements[key]) {
            if (key.indexOf('data-') == 0 || key.indexOf('class-') == 0 || key.indexOf('element-') == 0) {
                delete elements[key];
            }
            else {
                css += '\n        ' + key + ':' + elements[key] + ';';
            }
        }
    }
    return css;
}
exports.getCss = getCss;
function removeDupStyles(styles) {
    var values = [];
    var valueKeys = {};
    for (var key in styles) {
        var value = JSON.stringify(styles[key]);
        if (values.indexOf(value) == -1) {
            valueKeys[value] = "" + key;
            values.push(value);
        }
        else {
            valueKeys[value] = valueKeys[value] + (", ." + key);
        }
    }
    return valueKeys;
}
exports.removeDupStyles = removeDupStyles;
function capitalizeFirstLetter(word) {
    return word.charAt(0).toUpperCase() + word.slice(1);
}
exports.capitalizeFirstLetter = capitalizeFirstLetter;
function getComponentStyles(style, elements) {
    var compStyle = {
        large: [],
        medium: [],
        small: []
    };
    elements.forEach(function (e) {
        if (style.large[e]) {
            compStyle.large[e] = style.large[e];
            delete style.large[e];
        }
        if (style.medium[e]) {
            compStyle.medium[e] = style.medium[e];
            delete style.medium[e];
        }
        if (style.small[e]) {
            compStyle.small[e] = style.small[e];
            delete style.small[e];
        }
    });
    return compStyle;
}
exports.getComponentStyles = getComponentStyles;
