"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function getCss(elements) {
    var css = "";
    for (var key in elements) {
        if (key && elements[key]) {
            if (key.startsWith("data-") ||
                key.startsWith("class-") ||
                key.startsWith("element-")) {
                delete elements[key];
            }
            else {
                css += "\n        " + key + ":" + elements[key] + ";";
            }
        }
    }
    return css;
}
exports.getCss = getCss;
function capitalizeFirstLetter(word) {
    return word.charAt(0).toUpperCase() + word.slice(1);
}
exports.capitalizeFirstLetter = capitalizeFirstLetter;
function getComponentStyles(style, elements) {
    var compStyle = {
        large: [],
        medium: [],
        small: []
    };
    elements.forEach(function (e) {
        if (style.large[e]) {
            compStyle.large[e] = style.large[e];
            delete style.large[e];
        }
        if (style.medium[e]) {
            compStyle.medium[e] = style.medium[e];
            delete style.medium[e];
        }
        if (style.small[e]) {
            compStyle.small[e] = style.small[e];
            delete style.small[e];
        }
    });
    return compStyle;
}
exports.getComponentStyles = getComponentStyles;
function firstLetterUpper(word) {
    return word.charAt(0).toUpperCase() + word.substr(1);
}
exports.firstLetterUpper = firstLetterUpper;
function generateWebStyle(style, screen, classes) {
    var css = generateWebCss(style.large, screen, classes) + "\n";
    if (style.medium) {
        var code = generateWebCss(style.medium, screen, classes);
        if (code && code.trim() !== "") {
            css =
                css +
                    ("@media only screen and (max-width: 1024px)  {\n            " + code + "\n        }\n");
        }
    }
    if (style.small) {
        var code = generateWebCss(style.small, screen, classes);
        if (code && code.trim() !== "") {
            css =
                css +
                    ("@media only screen and (max-width: 540px)  {\n                " + code + "\n            }\n");
        }
    }
    return css;
}
exports.generateWebStyle = generateWebStyle;
function generateWebCss(styles, screen, classes) {
    var csscode = "";
    for (var key in styles) {
        var style = styles[key];
        if (classes[key] != undefined && classes[key] != "") {
            key = classes[key];
        }
        if (screen.find("." + key)[0]) {
            csscode += "const " + key + " {\n";
            csscode += getCss(style);
            csscode += "}\n";
        }
    }
    return csscode;
}
function getInlineCss(elements) {
    var css = "";
    for (var key in elements) {
        if (key && elements[key]) {
            if (key.startsWith("data-") ||
                key.startsWith("class-") ||
                key.startsWith("element-")) {
                delete elements[key];
            }
            else {
                css += "\n        " + inlineKey(key) + ":'" + elements[key] + "',";
            }
        }
    }
    return css;
}
exports.getInlineCss = getInlineCss;
function inlineKey(key) {
    if (!key.includes("-"))
        return key;
    var keys = key.split("-");
    key = keys.shift();
    for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
        var k = keys_1[_i];
        key += firstLetterUpper(k);
    }
    return key;
}
