import * as beautify from 'js-beautify';
import { ElementTypes } from '../../../data/values';
import { createFile } from '../../file';
import { $ } from '../../jq';
import { styleSheet } from './style-sheet';

export const generateReactNativeApp = async (compDir, project, component) => {
  if (component) {
    project.screens.forEach(screen => {
      if (screen.name === component) {
        let web = generateReactNative(screen.name, screen.html, screen.style);
        createFile(compDir, `${screen.name}.js`, web);
      }
    });
  } else {
    project.screens.forEach(screen => {
      let web = generateReactNative(screen.name, screen.html, screen.style);
      createFile(compDir, `${screen.name}.js`, web);
    });
  }
};

function generateReactNative(screenId, div, styles) {
  let screen = $(`<div>${div}</div>`);
  for (let key in styles) {
    if (styles[key]['element-type']) {
      let type = styles[key]['element-type'];
      if (type == ElementTypes.Text || type == ElementTypes.Email) {
        let span = $(`.${key} span`).first();
        let placeholder = '';
        if (span) {
          placeholder = span.text();
          span.remove();
        }
        let tag = `<TextInput class="${key}" placeholder='#${placeholder}#'>`;
        $(`.${key}`).replaceWith(tag);
      } else if (type == ElementTypes.Password) {
        let span = $(`.${key} span`).first();
        let placeholder = '';
        if (span) {
          placeholder = span.text();
          span.remove();
        }
        let tag = `<TextInput class="${key}" secureTextEntry="#" placeholder='#${placeholder}#'>`;
        $(`.${key}`).replaceWith(tag);
      } else if (type == ElementTypes.Button) {
        $(`.${key}`).replaceWith(`<TouchableHighlight class="${key}" >`);
      }
    }
  }

  let html = screen.html();
  html = html.replace(/="#"/g, '');
  html = html.replace(/"#/g, "'");
  html = html.replace(/#"/g, "'");
  html = html.replace(/(data)(.+?)(")/g, '"');
  html = html.replace(/class="/g, 'style={styles.');
  html = html.replace(/src=""/g, 'source={}');
  html = html.replace(/"/g, '}');
  html = html.replace(/div/g, 'View');
  html = html.replace(/span/g, 'Text');
  html = html.replace(/img/g, 'Image');
  html = html.replace(/(source={} >)/g, '/>');
  html = html.replace(/style=""/g, '');
  html = html.replace(/textinput/g, 'TextInput');
  html = html.replace(/touchablehighlight/g, 'TouchableHighlight');
  html = beautify.html(html);
  let react = "import React from 'react';\n";
  react += "import { View,Text,StyleSheet,Image,TextInput,TouchableHighlight } from 'react-native';\n";
  let css = generateNativeStyle(styles);
  react += `${css}\n`;
  css = beautify.css(css);
  react += `const ${screenId}Screen = () => {\n`;
  react += `return (\n`;
  react += `${html}\n`;
  react += `);\n`;
  react += `}\n`;
  react += `export default ${screenId}Screen;\n`;

  return react;
}
function generateNativeStyle(styles) {
  let csscode = 'const styles = StyleSheet.create({\n';
  for (var key in styles) {
    csscode += `${key}:{\n`;
    csscode += getReactNativeStyle(styles[key]);
    csscode += '},\n';
  }
  csscode += '});\n';
  csscode = csscode.replace(/undefined/g, '');
  return csscode;
}

function getReactNativeStyle(elements) {
  let css = '';
  for (var key in elements) {
    if (key && elements[key]) {
      let style = getReactNativeStyleElement(key, elements[key]);
      if (style) {
        css += `    ${style}`;
      }
    }
  }
  return css;
}

function getReactNativeStyleElement(index, value) {
  let key = getReactNativeKey(index);
  if (key === 'display' || key === 'order') return undefined;
  else if (key === 'flex') {
    return key + ':' + value.charAt(0) + ',\n';
  } else if (styleSheet[key] !== undefined) {
    if (styleSheet[key] === 'number') {
      let v = formatSize(value);
      if (v) {
        return `${key}:${formatSize(value)},\n`;
      }
      return undefined;
    }
    if (styleSheet[key] === 'string') {
      if (value) {
        return `${key}:'${value}',\n`;
      }
      return undefined;
    }
  }
  return undefined;
}

function getReactNativeKey(k) {
  let l = k.split('-');
  if (l.length === 1) {
    return k;
  } else {
    let s = l[0];
    for (let i = 1; i < l.length; i++) {
      s += l[i].charAt(0).toUpperCase() + l[i].slice(1);
    }
    return s;
  }
}
function formatSize(w: any): any {
  if (typeof w == 'number') return w;
  if (w.match(/px/)) return w.replace('px', '');
  else if (w.match(/%/)) return `"${w.replace('px', '%')}"`;
  else return w;
}
