import * as beautify from 'js-beautify';
import { ElementTypes } from '../../../data/values';
import { createDir, createFile, createImages } from '../../file';
import { $ } from '../../jq';
import { capitalizeFirstLetter, generateWebStyle } from '../style';

export const generateAngularApp = async (compDir, imgDir, project, component) => {
  if (component) {
    project.screens.forEach(screen => {
      if (screen.name === component) {
        let code = generateAngularHtml(screen, imgDir);
        createDir(`${compDir}/${screen.name}`);
        createFile(`${compDir}/${screen.name}`, `${screen.name}.css`, code.css);
        createFile(`${compDir}/${screen.name}`, `${screen.name}.html`, code.html);
        createFile(`${compDir}/${screen.name}`, `${screen.name}.js`, code.angular);
      }
    });
  } else {
    project.screens.forEach(screen => {
      let code = generateAngularHtml(screen, imgDir);
      createDir(`${compDir}/${screen.name}`);
      createFile(`${compDir}/${screen.name}`, `${screen.name}.css`, code.css);
      createFile(`${compDir}/${screen.name}`, `${screen.name}.html`, code.html);
      createFile(`${compDir}/${screen.name}`, `${screen.name}.js`, code.angular);
    });
  }
};

function generateAngularHtml(screen, imgDir) {
  let styles = screen.style;
  let screenId = screen.name;
  let div = screen.html;
  let element = $(`<div>${div}</div>`);
  createImages(element, imgDir, styles);
  for (let key in styles.large) {
    if (styles.large[key]['element-type']) {
      let type = styles.large[key]['element-type'];
      if (type == ElementTypes.Link) {
        element.find(`.${key}`).replaceWith(function(this) {
          return $(`<a class='${key}' href='/'/>`).append($(this).contents());
        });
      } else if (type == ElementTypes.Email || type == ElementTypes.Text || type == ElementTypes.Password) {
        let span = element.find(`.${key} span`).first();
        let placeholder = '';
        if (span) {
          placeholder = span.text();
          span.remove();
        }
        element.find(`.${key}`).replaceWith(function(this) {
          return $(`<input type='${type}' class='${key}' placeholder='${placeholder}' />`).append($(this).contents());
        });
      } else if (type == 'component') {
        let classes = [key];
        let elements = element.find(`.${key}`).find('*');

        for (let i = 0; i < elements.length; i++) {
          classes.push($(elements[i]).attr('class'));
        }
      }
    }

    element.find('img').replaceWith(function(this) {
      let src = $(this).attr('src');
      let name = $(this).attr('class');
      if (styles.large[name]) {
        let filename = styles.large[name]['data-filename'];
        if (src.includes('data:image/png')) {
          return $(`<img class='${$(this).attr('class')}' src='../assets/${filename}' />`);
        }
        if (src.includes('data:image/svg+xml')) {
          return $(`<img class='${$(this).attr('class')}' src='../assets/${filename}' />`);
        }
      }
      return '';
    });

    setClasses(element, screen.classes);

    let html = element.html();
    for (let key in styles.large) {
      if (styles.large[key]['element-type']) {
        let type = styles.large[key]['element-type'];
        if (type == ElementTypes.Component) {
          let lowerKey = key.toLowerCase();
          let apperKey = capitalizeFirstLetter(key);
          html = html.replace(new RegExp(`<${lowerKey}>`, 'g'), `<${apperKey}>`);
          html = html.replace(new RegExp(`</${lowerKey}>`, 'g'), `</${apperKey}>`);
        }
      }
    }

    let css = generateWebStyle(styles, element, screen.classes);
    html = element.html();
    html = beautify.html(html);
    css = beautify.css(css);
    let angular = createTs(screenId);
    angular = beautify.js(angular);
    return { angular, css, html };
  }
}

function setClasses(screen, classes) {
  let elements = screen.find('*');
  for (let i = 0; i < elements.length; i++) {
    let elementId = $(elements[i]).attr('class');
    if (classes && classes[elementId] !== undefined) {
      $(elements[i]).attr('class', classes[elementId]);
    }
  }
}

function createTs(screenId) {
  let angular = `
        import { Component } from '@angular/core';
        @Component({
            selector: 'app-${screenId}',
            templateUrl: './${screenId}.component.html',
            styleUrls: ['./${screenId}.component.css']
        })
        export class ${screenId}Component {
            
        }
    `;
  return angular;
}
