import * as beautify from 'js-beautify';
import { ElementTypes } from '../../../data/values';
import { createFile, createImages } from '../../file';
import { $ } from '../../jq';
import { generateWebStyle } from '../style';
export const generateWebApp = async (compDir, imgDir, project, component) => {
  if (component) {
    project.screens.forEach(screen => {
      if (screen.name === component) {
        generateWebHtml(screen, imgDir, compDir);
      }
    });
  } else {
    project.screens.forEach(screen => {
      generateWebHtml(screen, imgDir, compDir);
    });
  }
};

function generateWebHtml(screen, imgDir, compDir) {
  let styles = screen.style;
  let screenId = screen.name;
  let div = screen.html;
  let element = $(`<div>${div}</div>`);
  createImages(element, imgDir, styles);

  for (let id in styles.large) {
    if (styles.large[id]['element-type']) {
      let type = styles.large[id]['element-type'];
      if (type == ElementTypes.Link) {
        element.find(`.${id}`).replaceWith(function(this) {
          return $(`<a class='${id}' href='/'/>`).append($(this).contents());
        });
      } else if (type == ElementTypes.Email || type == ElementTypes.Text || type == ElementTypes.Password) {
        let span = element.find(`.${id} span`).first();
        let placeholder = '';
        if (span) {
          placeholder = span.text();
          span.remove();
        }
        element.find(`.${id}`).replaceWith(function(this) {
          return $(`<input type='${type}' class='${id}' placeholder='${placeholder}' />`).append($(this).contents());
        });
      }
    }
  }

  element.find('img').replaceWith(function(this) {
    let src = $(this).attr('src');
    let name = $(this).attr('class');
    if (styles.large[name]) {
      let filename = styles.large[name]['data-filename'];
      if (src.includes('data:image/png')) {
        return $(`<img class='${$(this).attr('class')}' src='./assets/${filename}' />`);
      }
      if (src.includes('data:image/svg+xml')) {
        return $(`<img class='${$(this).attr('class')}' src='./assets/${filename}' />`);
      }
    }
    return '';
  });

  setClasses(element, screen.classes);
  let html = `<html>
                <head>
                <title>${screenId}</title>
                <link rel = 'stylesheet'
                  type = 'text/css'
                  href = '${screenId}.css' />
                </head>
                <body>`;
  html += element.html();
  html += `</body>`;
  html += `</html>`;
  html = beautify.html(html);
  let css = generateWebStyle(styles, element, screen.classes);
  css = beautify.css(css);
  createFile(compDir, `${screenId}.css`, css);
  createFile(compDir, `${screenId}.html`, html);
}

function setClasses(screen, classes) {
  let elements = screen.find('*');
  for (let i = 0; i < elements.length; i++) {
    let elementId = $(elements[i]).attr('class');
    if (classes && classes[elementId] !== undefined) {
      $(elements[i]).attr('class', classes[elementId]);
    }
  }
}
