import * as beautify from 'js-beautify';
import * as _ from 'lodash';
import { ElementTypes } from '../../../../data/values';
import { createFile, createImages } from '../../../file';
import { $ } from '../../../jq';
import { firstLetterUpper, getCss } from '../../style';

export const generateEmotionReactApp = async (compDir, imgDir, project, component) => {
  if (component) {
    project.screens.forEach(screen => {
      if (screen.name === component) {
        let code = generateEmotionReact(screen, imgDir);
        createFile(compDir, `${screen.name}.js`, code);
      }
    });
  } else {
    project.screens.forEach(screen => {
      let code = generateEmotionReact(screen, imgDir);
      createFile(compDir, `${screen.name}.js`, code);
    });
  }
};

function generateEmotionReact(screen: any, imgDir) {
  let components = [];
  let $screen = _.cloneDeep(screen);
  let styles = $screen.style.large;
  let screenId = $screen.name;
  let div = $screen.html;
  let comp = $(`<div>${div}</div>`);
  createImages(comp, imgDir, styles);
  for (let key in styles) {
    if (styles[key]['element-type']) {
      let type = styles[key]['element-type'];
      if (type == ElementTypes.Link) {
        comp.find(`.${key}`).replaceWith(function(this) {
          return $(`<a class='${key}' href='/'/>`).append($(this).contents());
        });
      } else if (type == ElementTypes.Email || type == ElementTypes.Text || type == ElementTypes.Password) {
        let span = comp.find(`.${key} span`).first();
        let spanClass = span.attr('class');
        let placeholder = '';
        if (span) {
          placeholder = span.text();
          span.remove();
        }
        delete styles[spanClass];
        comp.find(`.${key}`).replaceWith(function(this) {
          return $(`<input type='${type}' class='${key}' placeholder='${placeholder}' #tag/>`).append(
            $(this).contents(),
          );
        });
      } else if (type == ElementTypes.Component) {
        let classes = [key];
        let elements = comp.find(`.${key}`).find('*');

        for (let i = 0; i < elements.length; i++) {
          classes.push($(elements[i]).attr('class'));
        }
      }
    }
  }
  $screen.style.large = styles;
  let { html, css } = transform(comp, $screen.style, $screen.classes, components);
  let react = createJs(html, screenId, css);
  return react;
}

function transform(comp: JQuery<HTMLElement>, style: any, classes: any, components = null) {
  comp.find('img').replaceWith(function(this) {
    let src = $(this).attr('src');
    let name = $(this).attr('class');
    let filename = style.large[name]['data-filename'];
    if (src.includes('data:image/png')) {
      return $(`<img class="${$(this).attr('class')}" src="../assets/${filename}"  alt=""  #tag/>`);
    }
    if (src.includes('data:image/svg+xml')) {
      return $(`<img class="${$(this).attr('class')}" src="../assets/${filename}"  alt=""  #tag/>`);
    }
    return '';
  });
  setClasses(comp, classes);
  let css = generateEmotionReactCss(style, comp, classes);

  for (let key in style.large) {
    comp.find(`.${key}`).replaceWith(function(this) {
      return $(`<${key}/>`).append($(this).contents());
    });
  }
  if (classes) {
    for (let key in classes) {
      let className = classes[key];
      comp.find(`.${className}`).replaceWith(function(this) {
        return $(`<${className}/>`).append($(this).contents());
      });
    }
  }
  let html = comp.html();
  if (classes) {
    for (let key in classes) {
      let className = classes[key];
      let lowerKey = _.cloneDeep(className).toLowerCase();
      html = html.replace(new RegExp(`<${lowerKey}>`, 'g'), `<${firstLetterUpper(className)}>`);
      html = html.replace(new RegExp(`</${lowerKey}>`, 'g'), `</${firstLetterUpper(className)}>`);
    }
  }

  for (let key in style.large) {
    let lowerKey = _.cloneDeep(key).toLowerCase();
    html = html.replace(new RegExp(`<${lowerKey}>`, 'g'), `<${firstLetterUpper(key)}>`);
    html = html.replace(new RegExp(`</${lowerKey}>`, 'g'), `</${firstLetterUpper(key)}>`);
  }

  if (components) {
    for (const key of components) {
      let lowerKey = _.cloneDeep(key).toLowerCase();
      html = html.replace(new RegExp(`<${lowerKey}>`, 'g'), `<${firstLetterUpper(key)}>`);
      html = html.replace(new RegExp(`</${lowerKey}>`, 'g'), `</${firstLetterUpper(key)}>`);
    }
  }

  html = html.replace(/#tag="">/g, '/>');
  html = html.replace(/href/g, 'to');
  html = beautify.html(html);
  css = beautify.css(css);
  return { html, css };
}

function setClasses(screen, classes) {
  let elements = screen.find('*');
  for (let i = 0; i < elements.length; i++) {
    let elementId = $(elements[i]).attr('class');
    if (classes && classes[elementId] !== undefined) {
      $(elements[i]).attr('class', classes[elementId]);
    }
  }
}

function createJs(html, screenId, css) {
  let react = `import React from 'react';\n`;
  react += `import styled from '@emotion/styled';\n`;
  react += `${css}\n`;
  react += `const ${firstLetterUpper(screenId)}Component = () => {\n`;
  react += `return (\n`;
  react += `${html}\n`;
  react += `);\n`;
  react += `}\n`;
  react += `export default ${firstLetterUpper(screenId)}Component;\n`;
  return react;
}

function generateEmotionReactCss(styles: any, screen: JQuery<HTMLElement>, classes: any): string {
  let csscode = '';
  let keys = [];
  for (let key in styles.large) {
    let style = styles.large[key];
    if (classes[key] != undefined && classes[key] != '') {
      key = classes[key];
    }
    if (screen.find(`.${key}`)[0]) {
      let tag = screen.find(`.${key}`)[0].tagName.toLowerCase();
      csscode += `const ${firstLetterUpper(key)} =styled.${tag}\`{\n`;
      csscode += getCss(style);
      if (styles.medium && styles.medium[key]) {
        csscode += `@media only screen and (max-width: 1024px)  {
          ${getCss(styles.medium[key])}
        }\n`;
      }
      if (styles.small && styles.small[key]) {
        csscode += `@media only screen and (max-width: 540px)  {
          ${getCss(styles.small[key])}
        }\n`;
      }
      csscode += '}`;\n';
    }

    keys.concat(key);
  }

  return csscode;
}
