import * as beautify from 'js-beautify';
import * as _ from 'lodash';
import { ElementTypes } from '../../../../data/values';
import { createFile, createImages } from '../../../file';
import { $ } from '../../../jq';
import { generateWebStyle } from '../../style';

export const generateReactModuleApp = async (compDir, imgDir, project, component) => {
  if (component) {
    project.screens.forEach(screen => {
      if (screen.name === component) {
        let code = generateReactModule(screen, imgDir);
        createFile(compDir, `${screen.name}.module.css`, code.css);
        createFile(compDir, `${screen.name}.js`, code.react);
      }
    });
  } else {
    project.screens.forEach(screen => {
      let code = generateReactModule(screen, imgDir);
      createFile(compDir, `${screen.name}.module.css`, code.css);
      createFile(compDir, `${screen.name}.js`, code.react);
    });
  }
};

function generateReactModule(screen: any, imgDir) {
  let components = [];
  let styles = screen.style;
  let screenId = screen.name;
  let div = screen.html;
  let comp = $(`<div>${div}</div>`);
  createImages(comp, imgDir, styles.large);

  for (let key in styles.large) {
    if (styles.large[key]['element-type']) {
      let type = styles.large[key]['element-type'];
      if (type == ElementTypes.Link) {
        comp.find(`.${key}`).replaceWith(function(this) {
          return $(`<a class='${key}' href='/'/>`).append($(this).contents());
        });
      } else if (type == ElementTypes.Email || type == ElementTypes.Text || type == ElementTypes.Password) {
        let span = comp.find(`.${key} span`).first();
        let spanClass = span.attr('class');
        let placeholder = '';
        if (span) {
          placeholder = span.text();
          span.remove();
        }
        if (styles.large && styles.large[spanClass]) delete styles.large[spanClass];
        if (styles.medium && styles.medium[spanClass]) delete styles.medium[spanClass];
        if (styles.small && styles.small[spanClass]) delete styles.small[spanClass];

        comp.find(`.${key}`).replaceWith(function(this) {
          return $(`<input type='${type}' class='${key}' placeholder='${placeholder}' #tag/>`).append(
            $(this).contents(),
          );
        });
      } else if (type == ElementTypes.Component) {
        let classes = [key];
        let elements = comp.find(`.${key}`).find('*');

        for (let i = 0; i < elements.length; i++) {
          classes.push($(elements[i]).attr('class'));
        }
      }
    }
  }

  let { html, css } = transform(comp, styles, screen.classes, components);
  let react = createJs(html, screenId);
  return { react, css };
}

function transform(comp: JQuery<HTMLElement>, styles: any, classes: any, components = null) {
  comp.find('img').replaceWith(function(this) {
    let src = $(this).attr('src');
    let name = $(this).attr('class');
    if (styles.large[name]) {
      let filename = styles.large[name]['data-filename'];
      if (src.includes('data:image/png')) {
        return $(`<img class="${$(this).attr('class')}" src="../assets/${filename}"  alt=""  #tag/>`);
      }
      if (src.includes('data:image/svg+xml')) {
        return $(`<img class="${$(this).attr('class')}" src="../assets/${filename}"  alt=""  #tag/>`);
      }
    }
    return '';
  });
  setClasses(comp, classes);
  let html = comp.html();
  for (let key in styles.large) {
    html = html.replace(new RegExp(`class="${key}"`, 'g'), `className={styles.${key}}`);
    html = html.replace(new RegExp(`class='${key}'`, 'g'), `className={styles.${key}}`);
  }
  if (classes) {
    for (let key in classes) {
      let className = classes[key];
      html = html.replace(new RegExp(`class="${className}"`, 'g'), `className={styles.${className}}`);
      html = html.replace(new RegExp(`class='${className}'`, 'g'), `className={styles.${className}}`);
    }
  }

  if (components) {
    for (const key of components) {
      let lowerKey = _.cloneDeep(key).toLowerCase();
      html = html.replace(new RegExp(`<${lowerKey}>`, 'g'), `<${key}>`);
      html = html.replace(new RegExp(`</${lowerKey}>`, 'g'), `</${key}>`);
    }
  }
  let css = generateWebStyle(styles, comp, classes);
  html = html.replace(/#tag="">/g, '/>');
  html = beautify.html(html);
  css = beautify.css(css);
  return { html, css };
}

function createReactComponent(name, htm, styles, classes, compDir) {
  let { html, css } = transform($(htm), styles, classes);
  let react = createJs(html, name);
  createFile(compDir, `${name}.module.css`, css);
  createFile(compDir, `${name}.js`, react);
}

function setClasses(screen, classes) {
  let elements = screen.find('*');
  for (let i = 0; i < elements.length; i++) {
    let elementId = $(elements[i]).attr('class');
    if (classes && classes[elementId] !== undefined) {
      $(elements[i]).attr('class', classes[elementId]);
    }
  }
}

function createJs(html, screenId) {
  let react = `import React from 'react';\n`;
  react += `import styles from './${screenId}.module.css';\n`;
  react += `const ${screenId} = () => {\n`;
  react += `return (\n`;
  react += `${html}\n`;
  react += `);\n`;
  react += `}\n`;
  react += `export default ${screenId};\n`;
  return react;
}
