import * as beautify from 'js-beautify';
import * as _ from 'lodash';
import { ElementTypes } from '../../../../data/values';
import { createFile, createImages } from '../../../file';
import { $ } from '../../../jq';
import { getInlineCss } from '../../style';

export const generateRadiumReactApp = async (compDir, imgDir, project, component) => {
  if (component) {
    project.screens.forEach(screen => {
      if (screen.name === component) {
        let code = generateRadiumReact(screen, imgDir);
        createFile(compDir, `${screen.name}.js`, code);
      }
    });
  } else {
    project.screens.forEach(screen => {
      let code = generateRadiumReact(screen, imgDir);
      createFile(compDir, `${screen.name}.js`, code);
    });
  }
};

function generateRadiumReact(screen: any, imgDir) {
  let components = [];
  let styles = screen.style.large;
  let screenId = screen.name;
  let div = screen.html;
  let comp = $(`<div>${div}</div>`);
  createImages(comp, imgDir, styles);
  for (let key in styles) {
    if (styles[key]['element-type']) {
      let type = styles[key]['element-type'];
      if (type == ElementTypes.Link) {
        comp.find(`.${key}`).replaceWith(function(this) {
          return $(`<a class='${key}' href='/'/>`).append($(this).contents());
        });
      } else if (type == ElementTypes.Email || type == ElementTypes.Text || type == ElementTypes.Password) {
        let span = comp.find(`.${key} span`).first();
        let spanClass = span.attr('class');
        let placeholder = '';
        if (span) {
          placeholder = span.text();
          span.remove();
        }
        delete styles[spanClass];

        comp.find(`.${key}`).replaceWith(function(this) {
          return $(`<input type='${type}' class='${key}' placeholder='${placeholder}' #tag/>`).append(
            $(this).contents(),
          );
        });
      } else if (type == ElementTypes.Component) {
        let classes = [key];
        let elements = comp.find(`.${key}`).find('*');

        for (let i = 0; i < elements.length; i++) {
          classes.push($(elements[i]).attr('class'));
        }
      }
    }
  }

  let { html, css } = transform(comp, screen.style, screen.classes, components);
  let react = createJs(html, screenId, css);
  return react;
}

function transform(comp: JQuery<HTMLElement>, styles: any, classes: any, components = null) {
  comp.find('img').replaceWith(function(this) {
    let src = $(this).attr('src');
    let name = $(this).attr('class');
    let filename = styles.large[name]['data-filename'];
    if (src.includes('data:image/png')) {
      return $(`<img class="${$(this).attr('class')}" src="../assets/${filename}"  alt=""  #tag/>`);
    }
    if (src.includes('data:image/svg+xml')) {
      return $(`<img class="${$(this).attr('class')}" src="../assets/${filename}"  alt=""  #tag/>`);
    }
    return '';
  });
  setClasses(comp, classes);
  let css = generateRadiumReactCss(styles, comp, classes);
  let html = comp.html();
  for (let key in styles.large) {
    html = html.replace(new RegExp(`class="${key}"`, 'g'), `style={styles.${key}}`);
    html = html.replace(new RegExp(`class='${key}'`, 'g'), `style={styles.${key}}`);
  }
  if (classes) {
    for (let key in classes) {
      let cls = classes[key];
      html = html.replace(new RegExp(`class="${cls}"`, 'g'), `style={styles.${cls}}`);
      html = html.replace(new RegExp(`class='${cls}'`, 'g'), `style={styles.${cls}}`);
    }
  }

  if (components) {
    for (const key of components) {
      let lowerKey = _.cloneDeep(key).toLowerCase();
      html = html.replace(new RegExp(`<${lowerKey}>`, 'g'), `<${key}>`);
      html = html.replace(new RegExp(`</${lowerKey}>`, 'g'), `</${key}>`);
    }
  }

  html = html.replace(/#tag="">/g, '/>');
  html = beautify.html(html);
  css = beautify(css);
  return { html, css };
}

function setClasses(screen, classes) {
  let elements = screen.find('*');
  for (let i = 0; i < elements.length; i++) {
    let elementId = $(elements[i]).attr('class');
    if (classes && classes[elementId] !== undefined) {
      $(elements[i]).attr('class', classes[elementId]);
    }
  }
}
function createJs(html, screenId, css) {
  let react = `import React from 'react';\n`;
  react += `import Radium from 'radium';\n`;
  react += `const ${screenId}Component = () => (\n`;
  react += `${html}\n`;
  react += `);\n`;
  react += `export default Radium(${screenId}Component);\n`;
  react += `const styles = {\n`;
  react += `${css}\n`;
  react += `};\n`;
  return react;
}

function generateRadiumReactCss(styles: any, screen: JQuery<HTMLElement>, classes: any): string {
  let csscode = '';
  for (let key in styles.large) {
    let style = styles.large[key];
    if (classes[key] != undefined && classes[key] != '') {
      key = classes[key];
    }
    if (screen.find(`.${key}`)[0]) {
      csscode += `${key} : {\n`;
      csscode += getInlineCss(style);
      if (styles.medium && styles.medium[key]) {
        csscode += `'@media only screen and (max-width: 1024px)':  {
          ${getInlineCss(styles.medium[key])}
        },\n`;
      }
      if (styles.small && styles.small[key]) {
        csscode += `'@media only screen and (max-width: 540px)':  {
          ${getInlineCss(styles.small[key])}
        }\n`;
      }
      csscode += '},\n';
    }
  }
  return csscode;
}
