export function getCss(elements: any): string {
  let css = "";
  for (let key in elements) {
    if (key && elements[key]) {
      if (
        key.startsWith("data-") ||
        key.startsWith("class-") ||
        key.startsWith("element-")
      ) {
        delete elements[key];
      } else {
        css += "\n        " + key + ":" + elements[key] + ";";
      }
    }
  }
  return css;
}

export function capitalizeFirstLetter(word: string): string {
  return word.charAt(0).toUpperCase() + word.slice(1);
}

export function getComponentStyles(style: any, elements: any): any {
  let compStyle = {
    large: [],
    medium: [],
    small: []
  };
  elements.forEach(function(e) {
    if (style.large[e]) {
      compStyle.large[e] = style.large[e];
      delete style.large[e];
    }
    if (style.medium[e]) {
      compStyle.medium[e] = style.medium[e];
      delete style.medium[e];
    }
    if (style.small[e]) {
      compStyle.small[e] = style.small[e];
      delete style.small[e];
    }
  });
  return compStyle;
}
export function firstLetterUpper(word) {
  return word.charAt(0).toUpperCase() + word.substr(1);
}

export function generateWebStyle(
  style: any,
  screen: JQuery<HTMLElement>,
  classes: any
): string {
  let css = `${generateWebCss(style.large, screen, classes)}\n`;
  if (style.medium) {
    let code = generateWebCss(style.medium, screen, classes);
    if (code && code.trim() !== "") {
      css =
        css +
        `@media only screen and (max-width: 1024px)  {
            ${code}
        }\n`;
    }
  }
  if (style.small) {
    let code = generateWebCss(style.small, screen, classes);
    if (code && code.trim() !== "") {
      css =
        css +
        `@media only screen and (max-width: 540px)  {
                ${code}
            }\n`;
    }
  }
  return css;
}
function generateWebCss(
  styles: any,
  screen: JQuery<HTMLElement>,
  classes: any
): string {
  let csscode = "";
  for (let key in styles) {
    let style = styles[key];
    if (classes[key] != undefined && classes[key] != "") {
      key = classes[key];
    }
    if (screen.find(`.${key}`)[0]) {
      csscode += `const ${key} {\n`;
      csscode += getCss(style);
      csscode += "}\n";
    }
  }
  return csscode;
}

export function getInlineCss(elements: any): string {
  let css = "";
  for (let key in elements) {
    if (key && elements[key]) {
      if (
        key.startsWith("data-") ||
        key.startsWith("class-") ||
        key.startsWith("element-")
      ) {
        delete elements[key];
      } else {
        css += "\n        " + inlineKey(key) + ":'" + elements[key] + "',";
      }
    }
  }
  return css;
}

function inlineKey(key: string) {
  if (!key.includes("-")) return key;
  let keys = key.split("-");
  key = keys.shift();
  for (let k of keys) {
    key += firstLetterUpper(k);
  }
  return key;
}
