import * as beautify from 'js-beautify';
import { ElementTypes } from '../../../data/values';
import { createFile, createImages } from '../../file';
import { $ } from '../../jq';
import { capitalizeFirstLetter, generateWebStyle } from '../style';

export const generateVueApp = async (compDir, imgDir, project, component) => {
  if (component) {
    project.screens.forEach(screen => {
      if (screen.name === component) {
        let web = generateVuetHtml(screen, imgDir);
        createFile(compDir, `${screen.name}.vue`, web);
      }
    });
  } else {
    project.screens.forEach(screen => {
      let web = generateVuetHtml(screen, imgDir);
      createFile(compDir, `${screen.name}.vue`, web);
    });
  }
};

function generateVuetHtml(screen, imgDir) {
  let styles = screen.style;
  let div = screen.html;
  let vue = '';
  let element = $(`<div>${div}</div>`);
  createImages(element, imgDir, styles);
  let imp = '';
  let comp = '';

  for (let key in styles.large) {
    if (styles.large[key]['element-type']) {
      let type = styles.large[key]['element-type'];
      if (type == ElementTypes.Link) {
        element.find(`.${key}`).replaceWith(function(this) {
          return $(`<a class='${key}' href='/'/>`).append($(this).contents());
        });
      } else if (type == ElementTypes.Email || type == ElementTypes.Text || type == ElementTypes.Password) {
        let span = element.find(`.${key} span`).first();
        let placeholder = '';
        if (span) {
          placeholder = span.text();
          span.remove();
        }
        element.find(`.${key}`).replaceWith(function(this) {
          return $(`<input type='${type}' class='${key}' placeholder='${placeholder}' />`).append($(this).contents());
        });
      } else if (type == ElementTypes.Component) {
        let classes = [key];
        let elements = element.find(`.${key}`).find('*');

        for (let i = 0; i < elements.length; i++) {
          classes.push($(elements[i]).attr('class'));
        }
      }
    }

    element.find('img').replaceWith(function(this) {
      let src = $(this).attr('src');
      let name = $(this).attr('class');
      if (styles.large[name]) {
        let filename = styles.large[name]['data-filename'];
        if (src.includes('data:image/png')) {
          return $(`<img class='${$(this).attr('class')}' src='../assets/${filename}' />`);
        }
        if (src.includes('data:image/svg+xml')) {
          return $(`<img class='${$(this).attr('class')}' src='../assets/${filename}' />`);
        }
      }
      return '';
    });

    setClasses(element, screen.classes);

    let html = element.html();
    for (let key in styles.large) {
      if (styles.large[key]['element-type']) {
        let type = styles.large[key]['element-type'];
        if (type == ElementTypes.Component) {
          let lowerKey = key.toLowerCase();
          let apperKey = capitalizeFirstLetter(key);
          html = html.replace(new RegExp(`<${lowerKey}>`, 'g'), `<${apperKey}>`);
          html = html.replace(new RegExp(`</${lowerKey}>`, 'g'), `</${apperKey}>`);
        }
      }
    }

    let css = generateWebStyle(styles, element, screen.classes);
    css = beautify.css(css);
    html = html.replace('</div>', '</div>\n');
    html = beautify.html(html);
    vue += `<template>\n${html}\n</template>\n`;
    vue += `<script>
            ${imp}       
            export default {
                components: {
                    ${comp}
                }
            }
        </script>\n`;
    vue += `<style scoped>\n${css}\n</style>\n`;

    return vue;
  }
  function setClasses(screen, classes) {
    let elements = screen.find('*');
    for (let i = 0; i < elements.length; i++) {
      let elementId = $(elements[i]).attr('class');
      if (classes && classes[elementId] !== undefined) {
        $(elements[i]).attr('class', classes[elementId]);
      }
    }
  }
}
