import { createDir } from '../code/file';
import { generateReactNativeApp } from '../code/mobile/reactnative/react-native';
import { generateAngularApp } from '../code/web/angular/angular';
import { generateWebApp } from '../code/web/html/html5';
import { generateReactApp } from '../code/web/react/generate-react';
import { generateVueApp } from '../code/web/vue/vue';
import { readProject } from '../data/project';
import { readSettings } from '../settings/project';
export const builder = yargs => {
  yargs
    .option('projectId', {
      describe: 'Enter the project Id',
      type: 'string',
    })
    .option('component', {
      describe: 'Enter the component name',
      type: 'string',
    })
    .option('type', {
      describe: 'Enter the project type:module,inline,styled,web,reactNative',
      type: 'string',
    })
    .option('style', {
      describe: 'Enter the style type(only for react projects) :module,inline,styled,aphrodite,emotion,jss,radium',
      type: 'string',
    })
    .help().argv;
};
export const handler = async function(args) {
  await generate(args);
  process.exit(0);
};

let generate = async function(args) {
  const settings = await getSettings(args);
  switch (settings.type.toLowerCase()) {
    case 'web':
      await generateWebApp(settings.compDir, settings.imgDir, settings.project, settings.component);
      break;
    case 'react':
      await generateReactApp(settings.compDir, settings.imgDir, settings.project, settings.component, settings.style);
      break;
    case 'reactnative':
      await generateReactNativeApp(settings.compDir, settings.project, settings.component);
      break;
    case 'vue':
      await generateVueApp(settings.compDir, settings.imgDir, settings.project, settings.component);
      break;
    case 'angular':
      await generateAngularApp(settings.compDir, settings.imgDir, settings.project, settings.component);
      break;
    default:
      process.exit(0);
  }
};
let getSettings = async args => {
  let imgDir: string, compDir: string, projectId: string, style: string, component: string, type: string;
  let dir = process.cwd();
  let settings = readSettings();
  projectId = settings && settings.projectId ? settings.projectId : args.projectId;
  type = settings && settings.type ? settings.type : args.type;
  style = settings && settings.style ? settings.style : args.style;
  if (settings && settings.imgDir) {
    imgDir = `${dir}/${settings.imgDir}`;
  } else {
    createDir(`${dir}/src`);
    imgDir = createDir(`${dir}/public/assets`);
  }
  if (settings && settings.compDir) {
    compDir = `${dir}/${settings.compDir}`;
  } else {
    createDir(`${dir}/src`);
    compDir = createDir(`${dir}/src/components`);
  }
  let project = await readProject(projectId);
  component = args.component;
  let ob = {
    project: project,
    style: style,
    component: component,
    type: type,
    compDir: compDir,
    imgDir: imgDir,
  };

  return ob;
};
