import Auth from '@aws-amplify/auth';
import Amplify from '@aws-amplify/core';
import { CognitoRefreshToken, CognitoUser, CognitoUserPool } from 'amazon-cognito-identity-js';
const Configstore = require('configstore');
const pkg = require('../../package.json');
const conf = new Configstore(pkg.name, { token: '', refreshToken: '', userId: '' });
const USER_POOL_ID = 'us-east-2_ixcAGkaCo';
const CLIENT_ID = '29of4a1bjtlbaqccroukh0l19i';

Amplify.configure({
  Auth: {
    identityPoolId: 'us-east-2:aa26ccf3-b3c3-4957-a59a-f7bdef3114d7',
    region: 'us-east-2',
    userPoolId: USER_POOL_ID,
    userPoolWebClientId: CLIENT_ID,
  },
});

let getCognitoUser = function(userName) {
  const poolData = {
    UserPoolId: USER_POOL_ID,
    ClientId: CLIENT_ID,
  };
  const userPool = new CognitoUserPool(poolData);
  const userData = {
    Username: userName,
    Pool: userPool,
  };
  return new CognitoUser(userData);
};

let getSession = function() {
  return {
    idToken: conf.get('token'),
    refreshToken: conf.get('refreshToken'),
    userName: conf.get('userName'),
  };
};

let setSession = function(session, userName) {
  conf.set('token', session.idToken.jwtToken);
  conf.set('refreshToken', session.refreshToken.token);
  conf.set('userName', userName);
};

let signIn = async function(email, password) {
  try {
    const user = await Auth.signIn(email, password);
    const session = await Auth.currentSession();
    setSession(session, user.username);
  } catch (err) {
    throw err;
  }
};

let generateSession = function() {
  let session = getSession();
  if (session.refreshToken !== undefined && session.refreshToken !== null) {
    const RefreshToken = new CognitoRefreshToken({ RefreshToken: session.refreshToken });
    let cognitoUser = getCognitoUser(session.userName);

    return new Promise(function(resolve, reject) {
      cognitoUser.refreshSession(RefreshToken, (err, data) => {
        if (err) {
          conf.clear();
          reject(err);
        } else {
          setSession(data, session.userName);
          resolve();
        }
      });
    });
  }
};

let refreshSession = async function() {
  await generateSession();
};

export { signIn, refreshSession };
